@@ -23,7 +23,7 @@
 #include "mx.h"
 #include "mxStack.h"
 
-#define VERSION "2.0.2"
+#define VERSION "2.0.3"
 
 /* The minimal size of stacks; must be > 1 */
 #define MINIMAL_STACKSIZE	4
@@ -81,8 +81,31 @@ PyObject *insexc(PyObject *moddict,
 {
     PyObject *v;
     char fullname[256];
+    char *modname;
+    char *dot;
     
-    sprintf(fullname,MXSTACK_MODULE".%s",name);
+    v = PyDict_GetItemString(moddict, "__name__");
+    if (v == NULL)
+	modname = NULL;
+    else
+	modname = PyString_AsString(v);
+    if (modname == NULL) {
+	PyErr_Clear();
+	modname = MXSTACK_MODULE;
+    }
+    /* The symbols from this extension are imported into
+       mx.<packagename>. We trim the name to not confuse the user with
+       an overly long package path. */
+    strcpy(fullname, modname);
+    dot = strchr(fullname, '.');
+    if (dot)
+	dot = strchr(dot+1, '.');
+    if (dot)
+	strcpy(dot+1, name);
+    else
+	sprintf(fullname, "%s.%s", modname, name);
+
+
     v = PyErr_NewException(fullname, base, NULL);
     if (v == NULL)
 	return NULL;
@@ -174,7 +197,7 @@ void mxStack_Free(register mxStackObject *stack)
 	    Py_DECREF(stack->array[i]);
 	free(stack->array);
     }
-    PyMem_DEL(stack);
+    PyObject_Del(stack);
 }
 
 /* --- internal functions --- */
