@@ -3,12 +3,13 @@
 """ Distutils Extensions needed for the mx Extensions.
 
     Copyright (c) 1997-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2003, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2004, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
 #
 # History:
+# 2.1.2: added wide Unicode support
 # 2.1.1: added support for Python 2.3 way of finding MSVC paths
 # 2.1.0: added bdist_zope, support for classifiers
 # 2.0.0: reworked the include and lib path logic; factored out the
@@ -103,23 +104,63 @@ def add_dir(dir, pathlist, index=-1):
             index = index + len(pathlist) + 1
         pathlist.insert(index, dir)
 
+def py_version(unicode_aware=1, include_patchlevel=0):
+
+    """ Return the Python version as short string.
+
+        If unicode_aware is true (default), the function also tests
+        whether a UCS2 or UCS4 built is running and modifies the
+        version accordingly.
+
+        If include_patchlevel is true (default is false), the patch
+        level is also included in the version string.
+
+    """
+    if include_patchlevel:
+        version = sys.version[:5]
+    else:
+        version = sys.version[:3]
+    if unicode_aware and version > '2.0':
+        try:
+            unichr(100000)
+        except ValueError:
+            pass
+        else:
+            # UCS4 build
+            version = version + 'ucs4'
+    return version
+
+def check_zope_product_version(version, version_txt):
+
+    """ Check whether the version string version matches the
+        version data in the Zope product version.txt file
+        version_txt.
+
+    """
+    data = open(version_txt, 'r').read().strip()
+    return data[-len(version):] == version
+
 def mx_customize_compiler(compiler):
 
     # Allow environment variables to override settings in the
     # Python Makefile on Unix; this overrides
     # distutils.sysconfig.customize_compiler()
     if compiler.compiler_type == "unix":
-        cc, opt, ccshared, ldshared, so = \
-             get_config_vars('CC', 'OPT', 'CCSHARED', 'LDSHARED', 'SO')
-        cc = os.environ.get('CC', cc)
-        opt = os.environ.get('OPT', opt)
-        ccshared = os.environ.get('CCSHARED', ccshared)
-        ldshared = os.environ.get('LDSHARED', ldshared)
-        so = os.environ.get('SO', so)
+        # Note: BASECFLAGS is new in Python 2.3; thanks to
+        # David M. Cooke for pointing this out to us.
+        cc, opt, ccshared, basecflags, ldshared, so = \
+             get_config_vars('CC', 'OPT', 'BASECFLAGS', 'CCSHARED',
+                             'LDSHARED', 'SO')
+        cc = os.environ.get('CC', cc or '')
+        opt = os.environ.get('OPT', opt or '')
+        basecflags = os.environ.get('BASECFLAGS', basecflags or '')
+        ccshared = os.environ.get('CCSHARED', ccshared or '')
+        ldshared = os.environ.get('LDSHARED', ldshared or '')
+        so = os.environ.get('SO', so or '')
         compiler.set_executables(
             preprocessor='%s -E' % cc,
-            compiler='%s %s' % (cc, opt),
-            compiler_so='%s %s %s' % (cc, opt, ccshared),
+            compiler='%s %s %s' % (cc, opt, basecflags),
+            compiler_so='%s %s %s %s' % (cc, opt, ccshared, basecflags),
             linker_so=ldshared,
             linker_exe=cc)
         compiler.shared_lib_extension = so
@@ -283,8 +324,10 @@ verify_path(STDINCLPATH)
 # Default paths for additional library and include file search (in
 # addition to the standard system directories above); these are always
 # added to the compile and link commands by mxSetup per default.
-LIBPATH = build_path(['/usr/local/lib', os.path.join(sys.prefix, 'lib')])
-INCLPATH = build_path(['/usr/local/include', os.path.join(sys.prefix, 'include')])
+LIBPATH = build_path(['/usr/local/lib',
+                      os.path.join(sys.prefix, 'lib')])
+INCLPATH = build_path(['/usr/local/include',
+                       os.path.join(sys.prefix, 'include')])
 
 verify_path(LIBPATH)
 verify_path(INCLPATH)
@@ -625,13 +668,20 @@ class mx_autoconf(CompilerSupportMixin,
 # here. This is an awful hack, but there seems to be no other way to
 # subclass a standard distutils C compiler class...
 
+if sys.version[:3] < '2.4':
+    # VC6
+    MSVC_COMPILER_FLAGS = ['/O2', '/Gf', '/GB', '/GD', '/Ob2']
+else:
+    # VC7
+    MSVC_COMPILER_FLAGS = ['/O2', '/GF', '/GB', '/Ob2']
+
 # remember old __init__
 old_MSVCCompiler__init__ = MSVCCompiler.__init__
 
 def mx_msvccompiler__init__(self, *args, **kws):
 
     apply(old_MSVCCompiler__init__, (self,) + args, kws)
-    self.compile_options.extend(['/O2','/Gf','/GB','/GD', '/Ob2'])
+    self.compile_options.extend(MSVC_COMPILER_FLAGS)
 
 # "Install" new __init__
 MSVCCompiler.__init__ = mx_msvccompiler__init__
@@ -1102,7 +1152,7 @@ class mx_build_py(build_py):
                                 "removing %s" % filename,
                                 verbose=verbose, dry_run=dry_run)
 
-    def get_outputs (self, include_bytecode=1):
+    def get_outputs(self, include_bytecode=1):
 
         if not self.without_source or \
            not include_bytecode:
@@ -1596,22 +1646,31 @@ class mx_bdist_rpm(bdist_rpm):
         return l
     
 #
-# mx Zope binary distribution command
+# mx in-place binary distribution command
 #
 
-class mx_bdist_zope(bdist_dumb):
+class mx_bdist_inplace(bdist_dumb):
 
     """ Build binary Zope product distribution.
     """
 
-    def finalize_options (self):
+    # Path prefix to use in the distribution (all files will be placed
+    # under this prefix)
+    dist_prefix = ''
+
+    user_options = bdist_dumb.user_options + [
+        ('dist-prefix=', None,
+         'path prefix the binary distribution with'),
+        ]
+
+    def finalize_options(self):
 
         # Default to ZIP as format on all platforms
         if self.format is None:
             self.format = 'zip'
         # Default to <platform>-py<version> on all platforms
         if self.plat_name is None:
-            self.plat_name = '%s-py%s' % (get_platform(), sys.version[:3])
+            self.plat_name = '%s-py%s' % (get_platform(), py_version())
         bdist_dumb.finalize_options(self)
 
     # Hack to reuse bdist_dumb for our purposes; .run() calls
@@ -1621,10 +1680,23 @@ class mx_bdist_zope(bdist_dumb):
         cmdobj = bdist_dumb.reinitialize_command(self, command,
                                                  reinit_subcommands)
         if command == 'install':
-            cmdobj.install_lib = 'lib/python'
-            cmdobj.install_data = 'lib/python'
+            cmdobj.install_lib = self.dist_prefix
+            cmdobj.install_data = self.dist_prefix
         return cmdobj
 
+#
+# mx Zope binary distribution command
+#
+
+class mx_bdist_zope(mx_bdist_inplace):
+
+    """ Build binary Zope product distribution.
+    """
+
+    # Path prefix to use in the distribution (all files will be placed
+    # under this prefix)
+    dist_prefix = 'lib/python'
+
 ###
 
 def run_setup(configurations):
@@ -1667,6 +1739,7 @@ def run_setup(configurations):
                   'uninstall': mx_uninstall,
                   'bdist_rpm': mx_bdist_rpm,
                   'bdist_zope': mx_bdist_zope,
+                  'bdist_inplace': mx_bdist_inplace,
                   }
     if bdist_ppm is not None:
         extensions['bdist_ppm'] = bdist_ppm.bdist_ppm
